'use strict';

Object.defineProperty(exports, '__esModule', {
    value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _fsHelper = require('./fs-helper');

var _fsHelper2 = _interopRequireDefault(_fsHelper);

var _wsServer = require('./ws-server');

var _wsServer2 = _interopRequireDefault(_wsServer);

var _confText = require('../../conf/text');

var _confText2 = _interopRequireDefault(_confText);

var _confEvents = require('../../conf/events');

var _confEvents2 = _interopRequireDefault(_confEvents);

var fsEvents = _confEvents2['default'].FS.REMOTE,
    wsEvents = _confEvents2['default'].WS.REMOTE;

function startRemote(opts) {
    if (!_lodash2['default'].isNumber(opts.port)) return console.info(_confText2['default'].REMOTE_MISSING_PORT);
    if (!_lodash2['default'].isString(opts.secret)) return console.info(_confText2['default'].REMOTE_MISSING_SECRET);

    var log = _lodash2['default'].partial(console.info.bind(console), opts.secret);
    var receivedLog = _lodash2['default'].partial(log, '<');
    var errorLog = _lodash2['default'].partial(log, 'ERR');

    var wss = new _wsServer2['default']({
        port: opts.port,
        secret: opts.secret,
        debug: opts.debug,
        encrypt: opts.encrypt
    });

    var fsHelper = new _fsHelper2['default']();

    // WS events
    wss.on(wsEvents.UNAUTHORIZED, function () {
        log(_confText2['default'].SERVER_ON_UNAUTHORIZED);
        process.exit();
    });

    wss.on(wsEvents.CONNECTION_CLOSED, function () {
        log(_confText2['default'].SERVER_ON_CONNECTION_CLOSED);
        process.exit();
    });

    wss.on(wsEvents.FILE_CHANGE, function (message) {
        switch (message.changeType) {
            case 'add':
                fsHelper.addFile(message);
                break;
            case 'addDir':
                fsHelper.addDir(message);
                break;
            case 'change':
                fsHelper.addFile(message);
                break;
            case 'unlink':
                fsHelper.removePath(message);
                break;
            case 'unlinkDir':
                fsHelper.removePath(message);
                break;
            default:
                break;
        }
    });

    // FS Events
    fsHelper.on(fsEvents.ADD_FILE, receivedLog);
    fsHelper.on(fsEvents.ADD_DIR, receivedLog);
    fsHelper.on(fsEvents.DELETE, receivedLog);

    fsHelper.on(fsEvents.ADD_FILE_ERROR, errorLog);
    fsHelper.on(fsEvents.ADD_DIR_ERROR, errorLog);
    fsHelper.on(fsEvents.DELETE_ERROR, errorLog);
};

exports['default'] = startRemote;
module.exports = exports['default'];