'use strict';

Object.defineProperty(exports, '__esModule', {
    value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _latestVersion = require('latest-version');

var _latestVersion2 = _interopRequireDefault(_latestVersion);

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _child_process = require('child_process');

var _os = require('os');

var _path = require('path');

var _untildify = require('untildify');

var _untildify2 = _interopRequireDefault(_untildify);

var _util = require('./util');

var _util2 = _interopRequireDefault(_util);

var _packageJson = require('../package.json');

var _confConstants = require('../conf/constants');

var _confConstants2 = _interopRequireDefault(_confConstants);

var _confText = require('../conf/text');

var _confText2 = _interopRequireDefault(_confText);

var now = Date.now(),
    updateInfo = _fsExtra2['default'].existsSync(_util2['default'].getUpdatePath()) ? require(_util2['default'].getUpdatePath()) : {
    lastChecked: 0,
    version: _packageJson.version
};

var getLatestVersion = _lodash2['default'].partial(_lodash2['default'].ary(_latestVersion2['default'], 2), 'sicksync');

function updateRemote(project) {
    var ssh = _util2['default'].shellIntoRemote(project.username + '@' + project.hostname);

    var updateRemoteOnce = _lodash2['default'].once(function (stdin) {
        stdin.write(_confConstants2['default'].UPDATE_CMD + '\n');
    });

    // Update devbox
    ssh.stdout.on('data', function (data) {
        var message = data.toString();

        if (_lodash2['default'].contains(message, 'sicksync@')) {
            console.info(project.hostname, _confText2['default'].UPDATE_SUCCESS);

            return ssh.kill('SIGINT');
        }

        if (_lodash2['default'].contains(message, 'ERR!')) {
            console.info(project.hostname, _confText2['default'].UPDATE_FAIL);

            return ssh.kill('SIGINT');
        }

        updateRemoteOnce(ssh.stdin);
    });
}

function updateLocal() {
    (0, _child_process.exec)(_confConstants2['default'].UPDATE_CMD, function (error, stdout, stderr) {
        if (!!error || _lodash2['default'].contains(stderr, 'ERR!')) {
            return console.info(_os.hostname, _confText2['default'].UPDATE_FAIL, error || stderr);
        }

        console.info((0, _os.hostname)(), _confText2['default'].UPDATE_SUCCESS);
    });
}

function update(config, opts) {
    if (opts.check) {
        return getLatestVersion(function (err, version) {
            if (err) return;
            console.info('Latest Version:', version);
            console.info('Current Version:', _packageJson.version);
        });
    }

    if (opts.migrateConfig) {
        return migrateConfig(config);
    }

    _lodash2['default'].each(config.projects, updateRemote);

    updateLocal();
}

function notify() {
    if (updateInfo.version !== _packageJson.version) {
        return console.info(_confText2['default'].UPDATE_AVAILABLE, '\n', 'Current version:', _packageJson.version, '\n', 'Latest version:', updateInfo.version);
    }
}

function check() {
    if (now - updateInfo.lastChecked >= _confConstants2['default'].UPDATE_INTERVAL) {
        getLatestVersion(function (err, version) {
            if (err) return;
            _fsExtra2['default'].writeFileSync(_util2['default'].getUpdatePath(), JSON.stringify({
                version: version,
                lastChecked: now
            }));
        });
    }
}

function migrate1to2(config) {
    if (_lodash2['default'].isEmpty(config)) {
        var configLocation = (0, _untildify2['default'])(_confConstants2['default'].CONFIG_FILE_V1);
        config = require(configLocation);
        _fsExtra2['default']['delete'](configLocation);
    }

    config.project = (0, _path.basename)(config.sourceLocation);

    return {
        version: _packageJson.version,
        debug: config.debug,
        retryOnDisconnect: config.retryOnDisconnect,
        projects: [_lodash2['default'].chain(config).omit('debug', 'retryOnDisconnect').mapKeys(function (value, key) {
            if (key === 'userName') return 'username';
            return key;
        }).value()]
    };
}

function migrateConfig(config) {
    var configVersion = config.version ? config.version.replace(/\./g, '') : null;

    if (!configVersion) {
        _util2['default'].writeConfig(migrate1to2(config));
    }
}

exports['default'] = {
    migrate1to2: migrate1to2,
    migrateConfig: migrateConfig,
    update: update,
    notify: notify,
    check: check,
    updateLocal: updateLocal,
    updateRemote: updateRemote,
    getLatestVersion: getLatestVersion
};
module.exports = exports['default'];